@@ -9,6 +9,7 @@ import shutil
 import numpy as np
 import django
 from django.db import models
+from datetime import datetime
 import matplotlib.pyplot as plt
 from matplotlib.patches import Ellipse
 
@@ -24,7 +25,7 @@ from astropy.utils.data import clear_download_cache
 
 
 Run, Instance, Detection, Product, Source  = None, None, None, None, None
-SourceDetection, Comment, Tag, TagDetection, TagSourceDetection = None, None, None, None, None
+SourceDetection, Comment, Tag, TagSourceDetection = None, None, None, None
 
 
 # utils
@@ -72,7 +73,7 @@ def _write_products(products, prefix):
 # Connect to WALLABY database
 def connect():
     global Run, Instance, Detection, Product, Source
-    global SourceDetection, Comment, Tag, TagDetection, TagSourceDetection
+    global SourceDetection, Comment, Tag, TagSourceDetection
     os.environ["DJANGO_SECRET_KEY"] = "-=(gyah-@e$-ymbz02mhwu6461zv&1&8uojya413ylk!#bwa-l"
     os.environ["DJANGO_SETTINGS_MODULE"] = "api.settings"
     os.environ["DJANGO_ALLOW_ASYNC_UNSAFE"] = "True"
@@ -83,7 +84,7 @@ def connect():
     sys.path.append("/mnt/shared/wallaby/apps/SoFiAX_services/api/")
     django.setup()
     from tables.models import Run, Instance, Detection, Product, Source
-    from tables.models import SourceDetection, Comment, Tag, TagDetection, TagSourceDetection
+    from tables.models import SourceDetection, Comment, Tag, TagSourceDetection
     return
 
 
@@ -108,11 +109,18 @@ def get_catalog(tag):
     detection_field_names.remove("v_rad")
     detection_field_names.remove("l")
     detection_field_names.remove("b")
+    detection_field_names.remove("wm50")
     detection_field_names.remove("v_opt_peak")
     detection_field_names.remove("v_app_peak")
     detection_field_names.remove("v_rad_peak")
     detection_field_names.remove("l_peak")
     detection_field_names.remove("b_peak")
+    detection_field_names.remove("x_peak")
+    detection_field_names.remove("y_peak")
+    detection_field_names.remove("z_peak")
+    detection_field_names.remove("ra_peak")
+    detection_field_names.remove("dec_peak")
+    detection_field_names.remove("freq_peak")
     source_field_names = [field.name for field in Source._meta.fields if not isinstance(field, models.ForeignKey)]
     source_field_names.remove("id")
     
@@ -159,9 +167,18 @@ def get_catalog(tag):
 
     # Extract team release column
     team_release = tag.replace('DR', 'TR')
-    column_team_release = [team_release] * len(table)
+    column_team_release = [str(team_release)] * len(table)
     table.add_column(col=column_team_release, name='team_release')
-    
+
+    # Add additional properties
+    c = 299792.458
+    H0 = 70
+    f0 = 1.420405751768E9
+    dist_h = c * ((f0 / table['freq']) - 1.0) / H0
+    HI_mass = np.log10(49.7 * dist_h ** 2 * table['f_sum'])
+    table.add_column(col=dist_h, name='dist_h')
+    table.add_column(col=HI_mass, name='log_m_hi')
+
     return table
 
 
@@ -170,7 +187,7 @@ def save_catalog(tag, *args, **kwargs):
 
     """
     table = get_catalog(tag)
-    table.remove_columns(['comments', 'tags'])
+    table.remove_columns(['tags'])
     table.write(*args, **kwargs)
 
 
@@ -415,6 +432,13 @@ def save_spectrum(detection, filename):
     hdr['CRVAL1'] = detection.ra
     hdr['CRVAL2'] = detection.dec
     hdr['CRVAL3'] = detection.freq
+
+    # Provenance metadata
+    hdr['SBID'] = ''
+    hdr['SRCVER'] = ''
+    hdr['SRCTR'] = ''
+    hdr['DATE'] = datetime.now().isoformat()
+
     spectrum.close()
     return
 
@@ -435,6 +459,7 @@ def casda_deposit(table, deposit_name):
     os.mkdir(f'{deposit_name}/cubelets')
     os.mkdir(f'{deposit_name}/spectra')
     os.mkdir(f'{deposit_name}/moment_maps')
+    os.mkdir(f'{deposit_name}/plots')
 
     # Export products
     for row in table:
@@ -463,41 +488,56 @@ def casda_deposit(table, deposit_name):
 
         # TODO: add SDIBs to WALSBID header cards 
 
-    # Export catalogue
-    table.remove_column('id')
-    votable = from_table(table)
-    votable.version = '1.3'
+        # TODO: add moment 0 maps (png) to products under separate folder
+        mom0, _ = get_image(products.mom0)
+        plt.imshow(mom0)
+        plt.axis('off')
+        plt.savefig('%s/%s/%s_mom0.png' % (deposit_name, 'plots', filename_prefix))
+        plt.close()
 
-    table_columns = []
+    # Read columns and metadata
+    table_columns = {}
     module_path = os.path.dirname(os.path.abspath(__file__))
-    with open(f'{module_path}/ucd.csv', 'r') as file:
+    with open(f'{module_path}/source_column_metadata.csv', 'r') as file:
         reader = csv.reader(file)
         for row in reader:
-            name, ucd, units = row
+            name, ucd, units, description = row
             units = units.replace('pixel', 'pix')
             if units == '-':
                 units = None
-            table_columns.append((name, ucd, units))
+            table_columns[name] = (ucd, units, description)
 
+    # Export catalogue
+    column_names = [k for k in table_columns.keys()]
+    table.remove_column('id')
+    table.remove_column('tags')
+    table = table[column_names]
+    votable = from_table(table)
+    votable.version = '1.3'
+
+    # Update fields
     fields = []
-    for column in table_columns:
-        name, ucd, units = column
+    for name, meta in table_columns.items():
+        ucd, units, description = meta
         field = votable.get_field_by_id(name)
         field.ucd = ucd
         field.unit = units
+        field.description = description
         fields.append(field)
+        if field.datatype == 'unicodeChar':
+            field.datatype = 'char'
 
-    for resource in votable.resources:
-        # add params
-        resource.params.append(Param(votable, ID='catalogueName', name='Catalogue Name', value=deposit_name, arraysize='59'))
-        resource.params.append(Param(votable, ID='indexedFields', name='Indexed Fields', value='name,ra,dec,freq,f_sum,w20,team_release', arraysize='255'))
-        resource.params.append(Param(votable, ID='principalFields', name='Principal Fields', value='name,ra,dec,freq,f_sum,w20,team_release', arraysize='255'))
-        
+    for resource in votable.resources:        
         # update rows
         for t in resource.tables:
             for i, row in enumerate(fields):
                 t.fields[i] = row
 
+            # add params
+            t.params.append(Param(votable, ID='catalogueName', name='Catalogue Name', value=deposit_name, arraysize='59'))
+            t.params.append(Param(votable, ID='indexedFields', name='Indexed Fields', value='name,ra,dec,freq,f_sum,w20,team_release', arraysize='255'))
+            t.params.append(Param(votable, ID='principalFields', name='Principal Fields', value='name,ra,dec,freq,f_sum,w20,team_release', arraysize='255'))
+
     catalogue_file = f'{deposit_name}/catalogue/catalogue.xml'
     writeto(votable, catalogue_file)
 
